import { Component, ContentChildren, ElementRef, Input, QueryList, Renderer2, ViewChild } from '@angular/core';
import { IonMarker } from './ion-marker';
import { mapSettings } from '../config/google-maps.settings';
import { IonMapStyles } from '../config/maps.styles';
var IonStaticMapsComponent = (function () {
    function IonStaticMapsComponent(renderer) {
        this.renderer = renderer;
        /**
           * The height of the map. Default value is 400px.
           */
        this.height = 400;
        /**
           * The width of the map. Default value is 400px.
           */
        this.width = 400;
        /**
           * The zoom of the map. Default value is 15.
           */
        this.zoom = 15;
    }
    IonStaticMapsComponent.prototype.ngAfterViewInit = function () {
        // Set size to the element so it already fits the screen before loading
        this.renderer.setStyle(this.mapEl.nativeElement, 'height', this.height + "px");
        this.renderer.setStyle(this.mapEl.nativeElement, 'width', this.width + "px");
    };
    IonStaticMapsComponent.prototype.ngAfterContentInit = function () {
        // After content is rendered, load markers, if any
        this.markers = this.mapMarkers.toArray();
        // Then, generate the map itself
        this.mapsUrl = this.generateMapsUrl();
        this.mapEl.nativeElement.classList.add('loaded');
    };
    IonStaticMapsComponent.prototype.generateMapsUrl = function () {
        var latLng;
        if (this.lat && this.lng) {
            latLng = this.lat + "," + this.lng;
        }
        var _a = this, zoom = _a.zoom, width = _a.width, height = _a.height, mapType = _a.mapType, language = _a.language, format = _a.format;
        var style = this.parseMapStyles(this);
        var url = mapSettings.staticMapsUrl
            + 'key=' + mapSettings.apiKey
            + '&center=' + (latLng ? latLng : this.address)
            + '&zoom=' + zoom
            + '&size=' + (width + "x" + height)
            + (mapType ? ('&maptype=' + mapType) : '')
            + (language ? ('&language=' + language) : '')
            + (format ? ('&format=' + format) : '')
            + (style ? (this.convertMapStyles(style)) : '');
        url += this.buildMarkersUrl();
        // encoding is needed to build a valid url: https://developers.google.com/maps/web-services/overview#BuildingURLs
        return encodeURI(url);
    };
    IonStaticMapsComponent.prototype.buildMarkersUrl = function () {
        return this.mapMarkers
            .map(function (m) {
            var iconOrLabel = m.iconUrl ? "icon:" + m.iconUrl : "label:" + m.label;
            return "&markers=color:" + m.color + "|" + iconOrLabel + "|" + m.lat + "," + m.lng;
        })
            .reduce(function (x, y) { return x + y; });
    };
    IonStaticMapsComponent.prototype.parseMapStyles = function (map) {
        return typeof map.mapStyle === 'string'
            ? IonMapStyles[map.mapStyle]
            : map.mapStyle;
    };
    IonStaticMapsComponent.prototype.convertMapStyles = function (styles) {
        var extractStyleRules = function (styles) {
            return styles.map(function (rule) {
                var ruleElements = '';
                for (var s in rule) {
                    ruleElements += s + ":" + rule[s] + "|";
                }
                return ruleElements.substring(0, ruleElements.length);
            })
                .reduce(function (x, y) { return x + y; });
        };
        var styleStr = styles
            .map(function (style) {
            var url = '&style=';
            url += style.featureType ? 'feature:' + style.featureType + '|' : '';
            url += style.elementType ? 'element:' + style.elementType + '|' : '';
            url += extractStyleRules(style.stylers);
            return url.substring(0, url.length - 1);
        })
            .reduce(function (x, y) { return x + y; });
        // static maps url only takes hex color in the given format: 0xRRGGBB, so we have to convert
        styleStr = styleStr.replace(/#/g, '0x');
        return styleStr;
    };
    IonStaticMapsComponent.decorators = [
        { type: Component, args: [{
                    selector: 'ion-static-maps',
                    template: "\n    <img class=\"ion-maps\" #map [src]=\"mapsUrl\" alt=\"maps\"/>\n    <ng-content></ng-content>\n  ",
                    styles: ["\n    img {\n      background: #eee;\n      opacity: 0;\n      transition: opacity .15s ease-in;\n    }\n\n    img.loaded {\n        opacity: 1;\n    }\n  "]
                },] },
    ];
    /** @nocollapse */
    IonStaticMapsComponent.ctorParameters = function () { return [
        { type: Renderer2, },
    ]; };
    IonStaticMapsComponent.propDecorators = {
        "address": [{ type: Input },],
        "lat": [{ type: Input },],
        "lng": [{ type: Input },],
        "height": [{ type: Input },],
        "width": [{ type: Input },],
        "zoom": [{ type: Input },],
        "format": [{ type: Input },],
        "language": [{ type: Input },],
        "mapType": [{ type: Input },],
        "mapStyle": [{ type: Input },],
        "mapEl": [{ type: ViewChild, args: ['map',] },],
        "mapMarkers": [{ type: ContentChildren, args: [IonMarker,] },],
    };
    return IonStaticMapsComponent;
}());
export { IonStaticMapsComponent };
export var StaticIonMapType;
(function (StaticIonMapType) {
    StaticIonMapType["roadmap"] = "roadmap";
    StaticIonMapType["satellite"] = "satellite";
    StaticIonMapType["hybrid"] = "hybrid";
    StaticIonMapType["terrain"] = "terrain";
})(StaticIonMapType || (StaticIonMapType = {}));
export var StaticIonMapFormat;
(function (StaticIonMapFormat) {
    StaticIonMapFormat["png"] = "png";
    StaticIonMapFormat["jpeg"] = "jpeg";
    StaticIonMapFormat["gif"] = "gif";
})(StaticIonMapFormat || (StaticIonMapFormat = {}));
//# sourceMappingURL=ion-static-maps.js.map